package oclib

import (
	"encoding/json"

	"cloud.o-forge.io/core/oc-lib/models/resources"
	"cloud.o-forge.io/core/oc-lib/models/resources/data"
	"cloud.o-forge.io/core/oc-lib/models/resources/datacenter"
	"cloud.o-forge.io/core/oc-lib/models/resources/processing"
	"cloud.o-forge.io/core/oc-lib/models/resources/storage"
	"cloud.o-forge.io/core/oc-lib/models/resources/workflow/graph"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type AbstractWorkflow struct {
	Graph              graph.Graph                              `bson:"graph,omitempty" json:"graph,omitempty"`
	Datas              map[string]data.DataResource             `bson:"datas,omitempty" json:"datas,omitempty"`
	Storages           map[string]storage.StorageResource       `bson:"storages,omitempty" json:"storages,omitempty"`
	ProcessingResource map[string]processing.ProcessingResource `bson:"processing,omitempty" json:"processing,omitempty"`
	Datacenters        map[string]datacenter.DatacenterResource `bson:"datacenters,omitempty" json:"datacenters,omitempty"`
	Workflows          map[string]WorkflowResource              `bson:"workflows,omitempty" json:"workflows,omitempty"`
	Schedule           WorkflowSchedule                         `bson:"schedule,omitempty" json:"schedule,omitempty"`
}

func (w *AbstractWorkflow) ObjDeserialize(j map[string]interface{}) *AbstractWorkflow {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, w)
	return w
}

func (w *AbstractWorkflow) ObjSerialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(w)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, w)
	return m
}

func (w *AbstractWorkflow) isDCLink(link graph.GraphLink) bool {
	if _, exists := w.Datacenters[link.Destination.ID]; exists {
		return true
	} else if _, exists := w.Datacenters[link.Source.ID]; exists {
		return true
	}

	return false
}

type WorkflowResource struct {
	resources.AbstractResource
	AbstractWorkflow
	WorkflowID string `bson:"workflow_id,omitempty" json:"workflow_id,omitempty"`
}

func (d *WorkflowResource) GetAccessor() utils.Accessor {
	data := &WorkflowResourceMongoAccessor{}
	data.SetLogger(utils.WORKFLOW_RESOURCE)
	return data
}

func (dma *WorkflowResource) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	dma.AbstractResource.ObjDeserialize(j)
	return dma
}

func (dma *WorkflowResource) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	dma.AbstractResource.ObjSerialize()
	return m
}
